/***********************************************************
*                       pC/SFS 1.01b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                        pC_SFS.h                          *
*                 FILESYSTEM DECLARATIONS                  *
***********************************************************/
#ifndef _pC_SFS_
 #define _pC_SFS_


#include "SFS_cfg.h"                   // bind config of pC/SFS

/*
************************************************************
*                      ERROR CODES
************************************************************
*/
#define SFS_NO_ERR       0             // no errors

#define SFS_USR_OVF      200           // no user free
#define SFS_DBL_USER     201           // double user
#define SFS_NO_USER      202           // not a valid user
#define SFS_SUB_USER     203           // a user is registrated with this dir as ROOT

#define SFS_NAME_EXIST   210           // name of entry exist in this DIR
#define SFS_NOT_EXIST    211           // DIR or FILE not exist
#define SFS_PATH_ERR     212           // error on PATH
#define SFS_TMP_DIR      213           // Temp-Dir is set / not set
#define SFS_NO_FILE      214           // no FILE-Name given
#define SFS_FILE_RO      215           // file to open for writing is read-only
#define SFS_FILE_WO      216           // file to open for reading is write-only
#define SFS_FILE_EOF     217           // end-of-file while reading or writing
#define SFS_NOT_EMPTY    218           // entry not empty
#define SFS_FILE_OPEN    219           // current entry is opened file
#define SFS_NO_DATA      220           // current file-lenght is zero
#define SFS_WRONG_PTR    221           // offset into open file is wrong (or size for R/W)
#define SFS_NO_ENTRY     222           // no free entry in dir
#define SFS_WRONG_A      223           // wrong access flags or attributes given

#define SFS_LINKED       230           // entry is linked
#define SFS_MAX_LINK     231           // entry is max linked (15)
#define SFS_LINK_ERR     232           // error in link-mechanism
#define SFS_NO_LINK      233           // no link to an entry in link-entry

#define SFS_MEM_ERR      240           // error in memory-handle
#define SFS_MEM_OVF      241           // not enough memory

#define SFS_FORMAT_ERR   250           // any format difference or error 
#define SFS_PORT_ERR     251           // error from porting
/*
************************************************************
*                ENTRY-TYPES / FILE ATTRIBUTES
************************************************************
*/
#define SFS_ATTR_DIR     0x00          // Entry is a directory
#define SFS_ATTR_FILE    0x80          // Entry is a file
#define SFS_ATTR_LINK    0x40          // Entry is a link
#define SFS_ATTR_TYPE_M  0xC0          // Entry type mask

#define SFS_ATTR_RO      0x10          // Entry is read-only
#define SFS_ATTR_WO      0x20          // Entry is write-only
#define SFS_ATTR_RW      0x30          // Entry is read/write

#define SFS_ATTR_LINKS_M 0x0F          // count of links to this entry (mask)   
/*
************************************************************
*                      FILE ACCESS-MODES
************************************************************
*/
#define SFS_ACC_RO       SFS_ATTR_RO   // File will read only
#define SFS_ACC_WO       SFS_ATTR_WO   // File will written only
#define SFS_ACC_RW       SFS_ATTR_RW   // File will read and write

#define SFS_SEEK_EOF  (SFS_LONG)(0-1)  // goto EndOfFile

#ifdef OS_PACK_PRAGMA
 #pragma pack(1)
#endif
/*
***********************************************************
*                    SFS ENTRY STRUCTURE
***********************************************************
*/
OS_PACKED typedef struct sfs_e {       // 16 bytes default
    U32          name_hash;
    SFS_LONG     attr__block_cnt;
    SFS_LONG     block_start;
    SFS_LONG     bytes_used;
}  OS_PACKED_ATTR SFS_ENTRY;

/*
* attr__block_cnt:  high-byte is attr (dir/file/link + r/w)
*                   rest-bytes are block_cnt
*
* block_cnt:    on root entry - entries per every dir
*               on file entry - block count allocated
*               on link entry - index-number to entry in dir pointed by block_start
*               all other     - 0
*
* block_start:  on root entry - block count over all (incl. blocks_bitmap)
*               on link entry - number of start block of dir the linked entry is part of
*               on dir  entry - number of start block of the hirarchically prev or next dir block(s)
*               on file entry - number of start block of the data blocks
*
* bytes_used:   on root entry - size of a block in bytes
*               on file entry - count of used bytes, the allocated block count is in attr__block_cnt
*               all other     - 0
* 
* ROOT:  hash of "/"    |   DIR + block-cnt              |   blocks over all   |   block size
* back:  hash of ".."   |   DIR + 0                      |   block-start       |   0
* dir:   hash of "dir"  |   DIR + 0                      |   block-start       |   0
* file:  hash of "file" |   FILE + block-cnt             |   block-start       |   used_len
* link:  hash of "link" |   LINK + DIR/FILE + block-offs |   block-start       |   0
*
* the block-start of root is 0 - the begin of the used memory (the location of root itself).
* By using "blocks over all" and "block-size" in ROOT-entry we can calculate the 
* location of the block-manager (bit-mask area) at the end ot the used memory.
*
* The block-offs of links is the index-number of the linked entry in the dir pointed
* by the block-start.
* To avoid link following at every scan for a dir or file, the link entry itself gets 
* additional the attribute DIR/FILE from the linked entry at creation time.
*/

/*
***********************************************************
*                    OPENED FILE STRUCTURE
***********************************************************
*/
OS_PACKED typedef struct sfs_o {
    SFS_ENTRY    openEntry;            // copy of the opened file entry (not the file_blocks) : name_hash = 0 if not used
    SFS_LONG     o_entryPtr;           // complete pointer based on block of the dir, the open entry is located in and entry index (internal used)
    SFS_LONG     openOffset;           // r/w seek offset into the file_blocks in bytes
    U08          openMode;             // opened mode ro/wo/rw
} OS_PACKED_ATTR SFS_OPEN;

/*
***********************************************************
*                    SFS USER STRUCTURE
***********************************************************
*/
OS_PACKED typedef struct sfs_user {
    SFS_ENTRY    currentEntry;         // copy of the current dir entry (not the dir_blocks)
    SFS_LONG     h_entrybi;            // combination of block of the dir, the found entry is located in and entry index in high byte  (temp used)
#if (SFS_TempDIR != 0)
    SFS_ENTRY    TempDir;              // copy of the saved current dir entry (not the dir_blocks)
#endif
    SFS_OPEN     used[SFS_HANDLES];    // copies of the opened files entries (not the file_blocks)
    U08          ErrNo;                // ErrNo
    U08          Id;                   // thread-ID if a kernel is used
} OS_PACKED_ATTR SFS_USER;

/*
***********************************************************
*                    SFS GET STRUCTURE
***********************************************************
*/
OS_PACKED typedef struct sfs_get {
    U32          name_hash;
    SFS_ATTR     Attr;
    SFS_LONG     Length;
    U08          Linked;
} OS_PACKED_ATTR SFS_GET;

#ifdef OS_PACK_PRAGMA
 #pragma pack()
#endif

/*
***********************************************************
*                   SFS FUNCTION PROTOTYPES
***********************************************************
*/
U08          SFS_Init(void) KC51_REENTRANT;
U08 OS_HUGE *SFS_GetRev(void) KC51_REENTRANT;
U08          SFS_Format(void) KC51_REENTRANT;

U08          SFS_BecomeUser(SFS_USER OS_HUGE *SFSUser) KC51_REENTRANT;
U08          SFS_CloseUser(void) KC51_REENTRANT;

U08          SFS_CreateDir(U08 OS_HUGE *Name) KC51_REENTRANT;
U08          SFS_RemoveDir(U08 OS_HUGE *Name) KC51_REENTRANT;
U08          SFS_RenameDir(U08 OS_HUGE *OldName, U08 OS_HUGE *NewName) KC51_REENTRANT;
U08          SFS_ChangeDir(U08 OS_HUGE *Name) KC51_REENTRANT;

#if (SFS_TempDIR != 0)
U08          SFS_ChangeDirTemp(U08 OS_HUGE *Name) KC51_REENTRANT;
U08          SFS_BackDirTemp(void) KC51_REENTRANT;
#endif

U08          SFS_CreateFile(U08 OS_HUGE *Name, SFS_ATTR Attribs, SFS_LONG size) KC51_REENTRANT;
U08          SFS_RemoveFile(U08 OS_HUGE *Name) KC51_REENTRANT;
U08          SFS_AttribFile(U08 OS_HUGE *Name, SFS_ATTR Attribs) KC51_REENTRANT;
U08          SFS_RenameFile(U08 OS_HUGE *OldName, U08 OS_HUGE *NewName) KC51_REENTRANT;
SFS_ATTR     SFS_GetFileAttrib(U08 OS_HUGE *Name) KC51_REENTRANT;
SFS_LONG     SFS_GetFileSize(U08 OS_HUGE *Name) KC51_REENTRANT;
SFS_LONG     SFS_GetMaxFileSize(U08 OS_HUGE *Name) KC51_REENTRANT;

SFS_HANDLE   SFS_OpenFile(U08 OS_HUGE *Name, U08 Mode) KC51_REENTRANT;
U08          SFS_SeekFile(SFS_LONG offset, SFS_HANDLE handl) KC51_REENTRANT;
U08          SFS_SetEOF(SFS_HANDLE handl) KC51_REENTRANT;
SFS_LONG     SFS_TellFile(SFS_HANDLE handl) KC51_REENTRANT;
SFS_LONG     SFS_ReadFile(U08 OS_HUGE *dest, SFS_LONG size, SFS_HANDLE handl) KC51_REENTRANT;
SFS_LONG     SFS_WriteFile(U08 OS_HUGE *src, SFS_LONG size, SFS_HANDLE handl) KC51_REENTRANT;
U08          SFS_CloseFile(SFS_HANDLE handl) KC51_REENTRANT;

U08          SFS_GetEntry(U08 OS_HUGE *Name, SFS_GET OS_HUGE *get) KC51_REENTRANT;

#if (SFS_LINKS != 0)
U08          SFS_CreateLink(U08 OS_HUGE *OrgName, U08 OS_HUGE *Name) KC51_REENTRANT;
U08          SFS_RemoveLink(U08 OS_HUGE *Name) KC51_REENTRANT;
#endif

U08          SFS_GetErrNo(void) KC51_REENTRANT;

#if defined(_WIN32) || defined(WIN32) || defined(__LINUX__) || defined(__linux__) || defined(__CYGWIN__)
 S16         SFS_Flush(void);        // external function from Window/LINUX extender
#endif



/************************* END ****************************/
#endif
