/***********************************************************
*                       pC/SFS 1.01b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                    SFS_LLdriver_xRAM.c                   *
*               lowlevel parallel xRAM driver              *
*             (to split lowlevel & HW specific)            *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../inc/pC_SFS.h"

#if !defined(SFS_MEM_xRAM_Port)
 #error "wrong config for the RAM image SFS-port!"
#endif

#if defined(_WIN32) || defined(WIN32) || defined(__LINUX__) || defined(__linux__) || defined(__CYGWIN__)
 extern S16 SFSPort_Load();
#endif

/**********************************************************
 *   using an RAM / code array as memory
 *   (can use an image or work on const[] as r/o-filesystem)
 *
 *   SFS_MEMORY_START [ SFS_MEMORY_SIZE ]
 *
 **********************************************************/
#ifndef SFS_MEMORY_START
 U08 SFS_MEMORY_START[SFS_MEMORY_SIZE];
#endif

/*
************************************************************
*                     INIT RAM array
************************************************************
*/
U08 SFSPort_Init(void)
{
#if defined(_WIN32) || defined(WIN32) || defined(__LINUX__) || defined(__linux__)|| defined(__CYGWIN__)
    SFSPort_Load();
#endif
    return(SFS_NO_ERR);
}

/*
************************************************************
*                    READ FROM xRAM
************************************************************
*/
U08 SFSPort_Read(U32 offs, U08 OS_HUGE *dest, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG i = 0;

    sfsPort_lock();
    while(i < length) { dest[i] = ((U08 *)SFS_MEMORY_START)[offs + i]; i++; }
    sfsPort_unlock();
    return(SFS_NO_ERR);
}

/*
************************************************************
*         WRITE ALL OR PAGE INTO xRAM (internal)
************************************************************
*/
void sfsPortWriteP(U32 offs, U08 OS_HUGE *source, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG i = 0;

    sfsPort_lock();
#if defined(SFS_MEM_AT28LV010)
    ((U08 *)SFS_MEMORY_START)[0x5555] = 0xAA;         // write-sequence
    ((U08 *)SFS_MEMORY_START)[0x2AAA] = 0x55;
    ((U08 *)SFS_MEMORY_START)[0x5555] = 0xA0;
#endif
    while(i < length) { ((U08 *)SFS_MEMORY_START)[offs + i] = source[i]; i++; }
#if defined(SFS_MEM_AT28LV010)
    while((((U08 *)SFS_MEMORY_START)[offs + i - 1] & 0x80) != (source[i - 1] & 0x80));
#endif
    sfsPort_unlock();
}

/*
************************************************************
*                    WRITE INTO xRAM
************************************************************
*/
U08 SFSPort_Write(U32 offs, U08 OS_HUGE *source, SFS_LONG length) KC51_REENTRANT
{
#if (SFS_MEM_HW_PAGESIZE > 1)
    SFS_LONG s;

    while(length > 0) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        sfsPortWriteP(offs, source, s);
        source = &source[s];
        offs   += s;
        length -= s;
    }                                                      // up to page-size and read flag
#else
    sfsPortWriteP(offs, source, length);
#endif
    return(SFS_NO_ERR);
}

/*
************************************************************
*             FILL ALL OR PAGE INTO xRAM (internal)
************************************************************
*/
void sfsPortFillP(U32 offs, U08 value, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG i = 0;

    sfsPort_lock();
#if defined(SFS_MEM_AT28LV010)
    ((U08 *)SFS_MEMORY_START)[0x5555] = 0xAA;         // write-sequence
    ((U08 *)SFS_MEMORY_START)[0x2AAA] = 0x55;
    ((U08 *)SFS_MEMORY_START)[0x5555] = 0xA0;
#endif
    while(i < length) { ((U08 *)SFS_MEMORY_START)[offs + i] = value; i++; }
#if defined(SFS_MEM_AT28LV010)
    while((((U08 *)SFS_MEMORY_START)[offs + i - 1] & 0x80) != (value & 0x80));
#endif
    sfsPort_unlock();
}

/*
************************************************************
*                        FILL xRAM
************************************************************
*/
U08 SFSPort_Fill(U32 offs, U08 value, SFS_LONG length) KC51_REENTRANT
{
#if (SFS_MEM_HW_PAGESIZE > 1)
    SFS_LONG s;

    while(length > 0) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        sfsPortFillP(offs, value, s);
        offs   += s;
        length -= s;
    }
#else
    sfsPortFillP(offs, value, length);
#endif
    return(SFS_NO_ERR);
}

/************************* END ****************************/
